function ihb_TalSpaceView3D(varargin)
%==================================================================================
% FORMAT ihb_View3D(action)
% action
%   create
%   newCluster
%   newAxial
%   newFrontal
%   newSaggital
%==================================================================================
% Function to create and manage 3D view window
%==================================================================================
%----------------------------------------------------------------------------------
%   07.04.01    Sergey Pakhomov
%   Last modified 22.05.01
%----------------------------------------------------------------------------------
%   05.10.01    Added check MATLAB version for handling transparency
%               Added menu Save as Picture
%----------------------------------------------------------------------------------
if nargin > 1 error('Too many arguments in ihb_View3D function'); end
if nargin == 0
    action = 'create';
else
    action = varargin{1};    
end
%----------------------------------------------------------------------------------
% Check existance of the 3D window and axes
%----------------------------------------------------------------------------------
hFig3D = findobj('Type', 'figure', 'Tag', 'ihb_TalSpace3D_fig');
hAxis3D = findobj('Type', 'axes', 'Tag', 'ihb_axesTalSpace3D');
%----------------------------------------------------------------------------------
% If hFig3D is empty and action not equal to 'create' then return
%----------------------------------------------------------------------------------
if isempty(hFig3D) & ~strcmp(action, 'create')
    return;
end
%----------------------------------------------------------------------------------
% Get default values
%----------------------------------------------------------------------------------
dflFileName = ihb_FileFolderName('dfl');
load(dflFileName, 'ihbdfl_renderer', 'ihbdfl_projection');
load(dflFileName, 'ihbdfl_bAxesIsClusterOnly');
load(dflFileName, 'ihbdfl_xLimD', 'ihbdfl_yLimD', 'ihbdfl_zLimD');
load(dflFileName, 'ihbdfl_bApplySmoothing');
load(dflFileName, 'ihbdfl_smooth_filter', 'ihbdfl_smooth_size', 'ihbdfl_smooth_sd');
load(dflFileName, 'ihbdfl_color_surface');
load(dflFileName, 'ihbdfl_transparency');
%----------------------------------------------------------------------------------
% Main action switch
%----------------------------------------------------------------------------------
switch action
%==================================================================================
% Create new figure
%==================================================================================
case 'create'
    %------------------------------------------------------------------------------
    % if already exists then make active and return
    %------------------------------------------------------------------------------
    if ~isempty(hFig3D)
        figure(hFig3D);
        return;
    end
    %------------------------------------------------------------------------------
    % Figure does not exist - create new one
    %------------------------------------------------------------------------------
	hFig3D = figure(...
		'Visible', 'off',...
		'NumberTitle', 'off',...
		'MenuBar', 'none',...
		'Name', '3D view of cluster',...
        'Renderer', ihbdfl_renderer,...
        'PaperPositionMode', 'auto',...
		'Tag', 'ihb_TalSpace3D_fig'...
		);
    ihb_TalSpaceView3D('newCluster');
    set(hFig3D, 'Visible', 'on');
    %------------------------------------------------------------------------------
    %     Set menu
    %------------------------------------------------------------------------------
	hSaveAsPictureMenu = uimenu('Label', 'Save ss Picture');
        uimenu(hSaveAsPictureMenu, 'Label', 'JPEG', 'Callback', 'ihb_SaveAsPictureCB(''JPEG'')');
        uimenu(hSaveAsPictureMenu, 'Label', 'PNG', 'Callback', 'ihb_SaveAsPictureCB(''PNG'')');
        uimenu(hSaveAsPictureMenu, 'Label', 'TIFF', 'Callback', 'ihb_SaveAsPictureCB(''TIFF'')');
%==================================================================================
% Process new cluster data
%==================================================================================
case 'newCluster'
    hMsg = msgbox('Preparing 3D View. Wait please...', 'non-modal');
    children = get(hMsg, 'children');
    set(children(2), 'Visible', 'off');
    drawnow
	%------------------------------------------------------------------------------
	% Get cluster's information
	%------------------------------------------------------------------------------
	hFigMain = findobj('Type', 'figure', 'Tag', 'ihb_TalSpaceMain_fig');
    set(hFig3D, 'Visible', 'off');
	clusters = getappdata(hFigMain, 'clusters');
    indClusterToView = getappdata(hFigMain, 'indClusterToView');
	cl = clusters(indClusterToView);
    %------------------------------------------------------------------------------
    % if axes already exist then delete them
    %------------------------------------------------------------------------------
    if ~isempty(hAxis3D)
        delete(hAxis3D);
    end
	%------------------------------------------------------------------------------
    % Create new axes object
	%------------------------------------------------------------------------------
    set(0,'CurrentFigure', hFig3D);
    if ihbdfl_bAxesIsClusterOnly == 1
        XLim = [cl.xMin cl.xMax];
        YLim = [cl.yMin cl.yMax];
        ZLim = [cl.zMin cl.zMax];
    else
        XLim = ihbdfl_xLimD;
        YLim = ihbdfl_yLimD;
        ZLim = ihbdfl_zLimD;
    end
    
	hAxis3D = axes(... 
        'Parent', hFig3D,...
        'Tag', 'ihb_axesTalSpace3D',...
        'XLim', XLim, 'YLim', YLim, 'ZLim', ZLim,...
        'Color','none',...
        'Box', 'on',...
        'XDir', 'reverse',...
        'YDir', 'reverse',...
        'Projection', ihbdfl_projection,...
        'DataAspectRatioMode', 'manual',...
        'DataAspectRatio', [1 1 1]...
        );
    
    if ihbdfl_bAxesIsClusterOnly == 1
        set(hAxis3D, 'XTick', [], 'YTick', [], 'ZTick', []);
    end

    set(hFig3D, 'CurrentAxes', hAxis3D);
	view([-150,32]);    % set initial camera position
	lighting phong;     % set lighting algorithm
	rotate3d on;        % enable 3D rotation
	%------------------------------------------------------------------------------
    % Create and/or adjust light position
	%------------------------------------------------------------------------------
    hLight = findobj('Type', 'light', 'Tag', 'ihb_lightTalSpace3D');
    cameraPos = get(hAxis3D, 'CameraPosition');
    if isempty(hLight)
	    hLight = light('Tag', 'ihb_lightTalSpace3D', 'Position', cameraPos);
    else
        set(hLight, 'Position', cameraPos);  
    end
	%------------------------------------------------------------------------------
    % set callback to light follow the camera
	%------------------------------------------------------------------------------
	set(hFig3D, 'WindowButtonUpFcn', 'ihb_View3DButtonUpFcn');
	%------------------------------------------------------------------------------
    % Smooth volume data
	%------------------------------------------------------------------------------
    if ihbdfl_bApplySmoothing == 1
        switch ihbdfl_smooth_filter
        case 'box'
            V = smooth3(cl.vTal, 'box', ihbdfl_smooth_size);
        case 'gaussian'
            V = smooth3(cl.vTal, 'gaussian', ihbdfl_smooth_size, ihbdfl_smooth_sd');
        end
    else
	    V = cl.vTal;
    end
	%------------------------------------------------------------------------------
    % Prepare volume data and create isosurface patch object
	%------------------------------------------------------------------------------
 	set(0,'CurrentFigure', hFig3D)
	set(hFig3D, 'CurrentAxes', hAxis3D);
	iv = cl.hThreshold;
	FV = isosurface(cl.xTal, cl.yTal, cl.zTal, V, cl.hThreshold);
	hPatch = patch(FV);
	isonormals(cl.xTal, cl.yTal, cl.zTal, cl.vTal, hPatch);
	%------------------------------------------------------------------------------
    % Check MATLAB version for handling transparency
	%------------------------------------------------------------------------------
    [v,d]=version;
    if (str2num(v(1)) < 6)
	    set(hPatch, 'Tag', 'ihb_surfaceCluster', 'FaceColor', ihbdfl_color_surface, 'EdgeColor', 'none');
    else
	    set(hPatch, 'Tag', 'ihb_surfaceCluster', 'FaceColor', ihbdfl_color_surface, 'EdgeColor', 'none', 'FaceAlpha', ihbdfl_transparency);
    end
    delete(hMsg);
    set(hFig3D, 'Visible', 'on');
	%------------------------------------------------------------------------------
    % Draw contours
	%------------------------------------------------------------------------------
    ihb_TalSpaceView3D('newAxial');
    ihb_TalSpaceView3D('newFrontal');
    ihb_TalSpaceView3D('newSaggital');
%----------------------------------------------------------------------------------
% Draw Axial section
%----------------------------------------------------------------------------------
case 'newAxial'
 	set(0,'CurrentFigure', hFig3D)
	set(hFig3D, 'CurrentAxes', hAxis3D);
    drawContour('Axial');
%----------------------------------------------------------------------------------
% Draw Frontal section
%----------------------------------------------------------------------------------
case 'newFrontal'
 	set(0,'CurrentFigure', hFig3D)
	set(hFig3D, 'CurrentAxes', hAxis3D);
    drawContour('Frontal');
%----------------------------------------------------------------------------------
% Draw Saggital section
%----------------------------------------------------------------------------------
case 'newSaggital'
 	set(0,'CurrentFigure', hFig3D)
	set(hFig3D, 'CurrentAxes', hAxis3D);
    drawContour('Saggital');
%----------------------------------------------------------------------------------
end
%==================================================================================
% Function to draw contour 
%==================================================================================
function drawContour(sectionType)
%----------------------------------------------------------------------------------
% Get default values
%----------------------------------------------------------------------------------
dflFileName = 'ihb_TalSpaceDfl.mat';
load(dflFileName, 'ihbdfl_bApplySmoothing');
load(dflFileName, 'ihbdfl_smooth_filter', 'ihbdfl_smooth_size', 'ihbdfl_smooth_sd');
load(dflFileName, 'ihbdfl_color_cntAx');
load(dflFileName, 'ihbdfl_color_cntFr');
load(dflFileName, 'ihbdfl_color_cntSag');
load(dflFileName, 'ihbdfl_line_width');
%------------------------------------------------------------------------------
% Get cluster's information
%------------------------------------------------------------------------------
hFigMain = findobj('Type', 'figure', 'Tag', 'ihb_TalSpaceMain_fig');
clusters = getappdata(hFigMain, 'clusters');
indClusterToView = getappdata(hFigMain, 'indClusterToView');
cl = clusters(indClusterToView);
iv = cl.hThreshold;
drawSymLS = getappdata(hFigMain, 'drawSymLS');
drawSymRS = getappdata(hFigMain, 'drawSymRS');
%------------------------------------------------------------------------------
% Smooth volume data
%------------------------------------------------------------------------------
if ihbdfl_bApplySmoothing == 1
    switch ihbdfl_smooth_filter
    case 'box'
        V = smooth3(cl.vTal, 'box', ihbdfl_smooth_size);
    case 'gaussian'
        V = smooth3(cl.vTal, 'gaussian', ihbdfl_smooth_size, ihbdfl_smooth_sd');
    end
else
    V = cl.vTal;
end
%------------------------------------------------------------------------------
% switch according section type
%------------------------------------------------------------------------------
switch sectionType
%------------------------------------------------------------------------------
case 'Axial'
    h = findobj('Type', 'patch', 'Tag', 'ihb_cntAxial');
    if ~isempty(h) delete(h); end
	hAx = getappdata(hFigMain, 'hAxAxial');
    z = getappdata(hAx, 'sliceDist');
	h = contourslice(cl.xTal, cl.yTal, cl.zTal, V, [], [], [z], [iv iv]);
	set(h, 'EdgeColor', ihbdfl_color_cntAx, 'LineWidth', ihbdfl_line_width, 'Tag', 'ihb_cntAxial');
%------------------------------------------------------------------------------
case 'Frontal'
    h = findobj('Type', 'patch', 'Tag', 'ihb_cntFrontal');
    if ~isempty(h) delete(h); end
	hAx = getappdata(hFigMain, 'hAxFront');
    y = getappdata(hAx, 'sliceDist');
	h = contourslice(cl.xTal, cl.yTal, cl.zTal, V, [], [y], [], [iv iv]);
	set(h, 'EdgeColor', ihbdfl_color_cntFr, 'LineWidth', ihbdfl_line_width, 'Tag', 'ihb_cntFrontal');
%------------------------------------------------------------------------------
case 'Saggital'
    if drawSymLS == 0
        h = findobj('Type', 'patch', 'Tag', 'ihb_cntSaggitalL');
        if ~isempty(h) delete(h); end
		hAx = getappdata(hFigMain, 'hAxSagL');
        x = getappdata(hAx, 'sliceDist');
        h = contourslice(cl.xTal, cl.yTal, cl.zTal, V, [x], [], [], [iv iv]);
        set(h, 'EdgeColor', ihbdfl_color_cntSag, 'LineWidth', ihbdfl_line_width, 'Tag', 'ihb_cntSaggitalL');
    end
    if drawSymRS == 0
        h = findobj('Type', 'patch', 'Tag', 'ihb_cntSaggitalR');
        if ~isempty(h) delete(h); end
		hAx = getappdata(hFigMain, 'hAxSagR');
        x = getappdata(hAx, 'sliceDist');
        h = contourslice(cl.xTal, cl.yTal, cl.zTal, V, [x], [], [], [iv iv]);
        set(h, 'EdgeColor', ihbdfl_color_cntSag, 'LineWidth', ihbdfl_line_width, 'Tag', 'ihb_cntSaggitalR');
    end
%------------------------------------------------------------------------------
end
